@@ -1,9 +1,11 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE ExistentialQuantification #-}
 {-# LANGUAGE Rank2Types #-}
 module Main where
 
 import           Imports
 import           Utils
+import           Data.Char                    (chr)
 import           Data.Word
 import           Data.ByteArray               (Bytes, ScrubbedBytes, ByteArray)
 import qualified Data.ByteArray          as B
@@ -12,6 +14,10 @@ import qualified Data.ByteArray.Parse    as Parse
 
 import qualified SipHash
 
+#ifdef WITH_FOUNDATION_SUPPORT
+import qualified Foundation as F
+#endif
+
 data Backend = BackendByte | BackendScrubbedBytes
     deriving (Show,Eq,Bounded,Enum)
 
@@ -52,6 +58,19 @@ testGroupBackends x l =
         , testGroup "ScrubbedBytes" (l withScrubbedBytesWitness)
         ]
 
+testShowProperty :: Testable a
+                 => String
+                 -> (forall ba . (Show ba, Eq ba, ByteArray ba) => (ba -> ba) -> ([Word8] -> String) -> a)
+                 -> TestTree
+testShowProperty x p =
+    testGroup x
+        [ testProperty "Bytes" (p withBytesWitness showLikeString)
+        , testProperty "ScrubbedBytes" (p withScrubbedBytesWitness showLikeEmptySB)
+        ]
+  where
+    showLikeString  l = show $ map (chr . fromIntegral) l
+    showLikeEmptySB _ = show (withScrubbedBytesWitness B.empty)
+
 base64Kats =
     [ ("pleasure.", "cGxlYXN1cmUu")
     , ("leasure.", "bGVhc3VyZS4=")
@@ -145,6 +164,11 @@ main = defaultMain $ testGroup "memory"
     , testGroupBackends "hashing" $ \witnessID ->
         [ testGroup "SipHash" $ SipHash.tests witnessID
         ]
+    , testShowProperty "showing" $ \witnessID expectedShow (Words8 l) ->
+          (show . witnessID . B.pack $ l) == expectedShow l
+#ifdef WITH_FOUNDATION_SUPPORT
+    , testFoundationTypes
+#endif
     ]
   where
     basicProperties witnessID =
@@ -186,4 +210,31 @@ main = defaultMain $ testGroup "memory"
              in B.any p b1 == any p l
         , testProperty "singleton b == pack [b]" $ \b ->
             witnessID (B.singleton b) == B.pack [b]
+        , testProperty "span" $ \x (Words8 l) ->
+            let c = witnessID (B.pack l)
+                (a, b) = B.span (== x) c
+             in c == B.append a b
+        , testProperty "span (const True)" $ \(Words8 l) ->
+            let a = witnessID (B.pack l)
+             in B.span (const True) a == (a, B.empty)
+        , testProperty "span (const False)" $ \(Words8 l) ->
+            let b = witnessID (B.pack l)
+             in B.span (const False) b == (B.empty, b)
         ]
+
+#ifdef WITH_FOUNDATION_SUPPORT
+testFoundationTypes = testGroup "Foundation"
+  [ testCase "allocRet 4 _ :: F.UArray Int8 === 4" $ do
+      x <- (B.length :: F.UArray F.Int8 -> Int) . snd <$> B.allocRet 4 (const $ return ())
+      assertEqual "" 4 x
+  , testCase "allocRet 4 _ :: F.UArray Int16 === 4" $ do
+      x <- (B.length :: F.UArray F.Int16 -> Int) . snd <$> B.allocRet 4 (const $ return ())
+      assertEqual "" 4 x
+  , testCase "allocRet 4 _ :: F.UArray Int32 === 4" $ do
+      x <- (B.length :: F.UArray F.Int32 -> Int) . snd <$> B.allocRet 4 (const $ return ())
+      assertEqual "" 4 x
+  , testCase "allocRet 4 _ :: F.UArray Int64 === 8" $ do
+      x <- (B.length :: F.UArray F.Int64 -> Int) . snd <$> B.allocRet 4 (const $ return ())
+      assertEqual "" 8 x
+  ]
+#endif
